package eu.chorevolution.vsb.manager;

//import eu.chorevolution.vsb.bc.generators.JarGenerator;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.net.URL;
import java.util.Arrays;
import java.util.Collections;
import java.util.EmptyStackException;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;
import java.util.Random;
import java.util.Scanner;

import javax.tools.JavaCompiler;
import javax.tools.JavaCompiler.CompilationTask;
import javax.tools.JavaFileObject;
import javax.tools.StandardJavaFileManager;
import javax.tools.StandardLocation;
import javax.tools.ToolProvider;

import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;
import org.json.simple.parser.ParseException;
import com.sun.codemodel.JBlock;
import com.sun.codemodel.JClass;
import com.sun.codemodel.JClassAlreadyExistsException;
import com.sun.codemodel.JCodeModel;
import com.sun.codemodel.JConditional;
import com.sun.codemodel.JDefinedClass;
import com.sun.codemodel.JExpr;
import com.sun.codemodel.JFieldRef;
import com.sun.codemodel.JFieldVar;
import com.sun.codemodel.JForLoop;
import com.sun.codemodel.JInvocation;
import com.sun.codemodel.JMethod;
import com.sun.codemodel.JMod;
import com.sun.codemodel.JPackage;
import com.sun.codemodel.JVar;

import eu.chorevolution.vsb.artifact.generators.WarGenerator;
import eu.chorevolution.vsb.bc.manager.BcManagerRestService;
import eu.chorevolution.vsb.bc.manager.VsbOutput;
import eu.chorevolution.vsb.gm.protocols.primitives.BcGmSubcomponent;
import eu.chorevolution.vsb.gm.protocols.rest.BcRestSubcomponent;
import eu.chorevolution.vsb.gm.protocols.soap.BcSoapGenerator;
import eu.chorevolution.vsb.gm.protocols.soap.BcSoapSubcomponent;
import eu.chorevolution.vsb.gmdl.tools.serviceparser.ServiceDescriptionParser;
import eu.chorevolution.vsb.gmdl.utils.BcConfiguration;
import eu.chorevolution.vsb.gmdl.utils.Constants;
import eu.chorevolution.vsb.gmdl.utils.PathResolver;
import eu.chorevolution.vsb.gmdl.utils.GmServiceRepresentation;
import eu.chorevolution.vsb.gmdl.utils.enums.ProtocolType;

public class VsbManager {

	private boolean STARTING_FROM_JAR = false;

	public VsbManager() {

		// Test if class is running from jar or from class file.
		if (getClass().getClassLoader().getResource("config.json").toString().startsWith("jar")) {

			STARTING_FROM_JAR = true;
		}
	}

	/********************************
	 * Public method to call
	 ***********************************************/

	/**
	 * Returns VsbOutput object which contains the generated 1 byte array of
	 * archive war and 1 array bytes of wsdl file
	 * <p>
	 *
	 * @param interfaceDescriptionPath
	 *            string path of a gidl or gmdl file
	 * @param busProtocol
	 *            the protocole type to use (SOAP, REST, ...)
	 * @param service_name
	 *            the name of the service will be generated
	 * @return VsbOutput object
	 */

	public VsbOutput generateWar(String interfaceDescriptionPath, ProtocolType busProtocol, String service_name) {

		service_name = deleteSpecialChar(service_name);
		Constants.service_name = service_name;
		VsbOutput vsbOutput = generate(interfaceDescriptionPath, busProtocol);
		vsbOutput.setinvaddr_url = Constants.setinvaddr_service_url;

		PathResolver.ouputInfo();
		return vsbOutput;
	}

	/**
	 * Returns VsbOutput object which contains the generated 1 byte array of
	 * archive war and 1 array bytes of wsdl file
	 * <p>
	 *
	 * @param bytes
	 *            array of a gidl or gmdl file
	 * @param busProtocol
	 *            the protocole type to use (SOAP, REST, ...)
	 * @param service_name
	 *            the name of the service will be generated
	 * @return VsbOutput object
	 */

	public VsbOutput generateWar(byte[] interfaceDescriptionByteArray, ProtocolType busProtocol, String service_name) {

		service_name = deleteSpecialChar(service_name);
		Constants.service_name = service_name;
		String interfaceDescriptionPath = interfaceDescriptionBytesToFile(interfaceDescriptionByteArray);
		VsbOutput vsbOutput = generate(interfaceDescriptionPath, busProtocol);
		vsbOutput.setinvaddr_url = Constants.setinvaddr_service_url;

		File interfaceDescriptionFile = new File("interfaceDescription.gidl");
		if (interfaceDescriptionFile.exists()) {

			interfaceDescriptionFile.delete();
		}
		PathResolver.ouputInfo();
		return vsbOutput;
	}

	/**
	 * Returns boolean, it deletes all generated temporary files
	 * <p>
	 * 
	 * @return boolean
	 */

	public boolean deleteGeneratedFiles() {

		boolean deleteGerated = false;
		File directory = new File(Constants.generatedCodePath);
		// make sure directory exists
		if (!directory.exists()) {

			System.out.println("Delete request failed: Directory does not exist.");
		} else {

			PathResolver.deleteTempDir(directory);
			deleteGerated = true;
		}
		return deleteGerated;
	}

	/******************************************************************************************************/

	/***************
	 * Private method called by public method
	 ************************************************/

	private void setConstants(String interfaceDescriptionPath) {

		Random rand = new Random();
		Constants.service_port = String
				.valueOf(rand.nextInt(Constants.service_port_max_range - Constants.service_port_min_range + 1)
						+ Constants.service_port_min_range);
		Constants.setinvaddr_service_port = String.valueOf(rand
				.nextInt(Constants.setinvaddr_service_port_max_range - Constants.setinvaddr_service_port_min_range + 1)
				+ Constants.setinvaddr_service_port_min_range);
		Constants.service_bc_port = String
				.valueOf(rand.nextInt(Constants.service_bc_port_max_range - Constants.service_bc_port_min_range + 1)
						+ Constants.service_bc_port_min_range);
		Constants.bc_manager_servlet_port = String.valueOf(rand
				.nextInt(Constants.bc_manager_servlet_port_max_range - Constants.bc_manager_servlet_port_min_range + 1)
				+ Constants.bc_manager_servlet_port_min_range);

		Constants.generatedCodePath = PathResolver.createTempDir();

		Class[] classe = { BcManagerRestService.class };
		Constants.configFilePath = PathResolver.myFilePath(BcManagerRestService.class, "config.json");

		Constants.interfaceDescriptionFilePath = interfaceDescriptionPath;
		String bc_generators_copy = Constants.generatedCodePath + File.separator + "bc-generators";

		// Test if class is running from jar or from class file.
		if (STARTING_FROM_JAR) {

			// URL webapp_src_bc = getClass().getClassLoader().getResource(
			// "webapp-bc-generator" );
			URL webapp_src_artifact = getClass().getClassLoader().getResource("artifact-generators");

			// PathResolver.extractDirectoryFromJar(webapp_src_bc,
			// bc_generators_copy);
			PathResolver.extractDirectoryFromJar(webapp_src_artifact, bc_generators_copy);
		} else {

			PathResolver.copyFolder(new File(".." + File.separator + "bc-generators").getAbsolutePath(),
					bc_generators_copy);
		}

		PathResolver.updatePortServiceSetInvAddr(Constants.setinvaddr_service_port);
		Constants.webapp_src_bc = new File(bc_generators_copy + File.separator + "webapp-bc-generator" + File.separator
				+ "src" + File.separator + "main" + File.separator + "webapp").getAbsolutePath();
		Constants.webapp_src_artifact = new File(bc_generators_copy + File.separator + "artifact-generators"
				+ File.separator + "src" + File.separator + "main" + File.separator + "webapp").getAbsolutePath();
		Constants.warName = "war" + Constants.service_name + "BC";
		Constants.wsdlName = "wsdl" + Constants.service_name + "BC";
		Constants.warDestination = new File(Constants.generatedCodePath + File.separator + Constants.warName + ".war")
				.getAbsolutePath();
		Constants.target_namespace = "eu.chorevolution.vsb.bindingcomponent.generated";
		Constants.target_namespace_path = Constants.target_namespace.replace(".", File.separator);
		Constants.soap_service_name = "BC" + Constants.service_name + "SoapEndpoint";
		// Constants.dpws_service_name =
		// "BC"+Constants.service_name+"DPWSEndpoint";
		Constants.wsdlDestination = new File(Constants.generatedCodePath).getAbsolutePath();
	}

	private VsbOutput generate(String interfaceDescriptionPath, ProtocolType busProtocol) {

		if (!isInterfaceDescriptionFile(interfaceDescriptionPath)) {

			System.err.println("Interface Description file not found or not good file extension");
			throw new EmptyStackException();
		}

		setConstants(interfaceDescriptionPath);

		generateBindingComponent(interfaceDescriptionPath, busProtocol);
		WarGenerator warGenerator = new WarGenerator();
		warGenerator.addPackage(eu.chorevolution.vsb.manager.VsbManager.class.getPackage());
		warGenerator.addPackage(eu.chorevolution.vsb.java2wsdl.Java2WSDL.class.getPackage());
		warGenerator
				.addPackage(eu.chorevolution.vsb.gmdl.tools.serviceparser.ServiceDescriptionParser.class.getPackage());
		
		warGenerator.addPackage(eu.chorevolution.vsb.gmdl.tools.serviceparser.gidl.ParseGIDL.class.getPackage());
		warGenerator.addPackage(eu.chorevolution.vsb.gmdl.tools.serviceparser.gmdl.ParseGMDL.class.getPackage());
		warGenerator.addPackage(eu.chorevolution.vsb.gmdl.utils.Operation.class.getPackage());
		warGenerator.addPackage(eu.chorevolution.vsb.gmdl.utils.enums.OperationType.class.getPackage());
		
		warGenerator.addPackage(eu.chorevolution.vsb.gm.protocols.coap.BcCoapGenerator.class.getPackage());
		warGenerator.addPackage(eu.chorevolution.vsb.gm.protocols.soap.BcSoapGenerator.class.getPackage());
		warGenerator.addPackage(eu.chorevolution.vsb.gm.protocols.rest.BcRestGenerator.class.getPackage());
		warGenerator.addPackage(eu.chorevolution.vsb.bc.manager.BcManagerRestService.class.getPackage());
		
		warGenerator.addPackage(eu.chorevolution.vsb.artifact.war.RestServlet.class.getPackage());
		warGenerator.addPackage(eu.chorevolution.vsb.artifact.generators.WarGenerator.class.getPackage());
		warGenerator.addPackage(eu.chorevolution.vsb.bc.setinvaddrservice.BaseService.class.getPackage());
		warGenerator.addPackage(eu.chorevolution.vsb.gm.protocols.Manageable.class.getPackage());
		
		String vsb_manager_pomxml = new File(".").getAbsolutePath() + File.separator + "pom.xml";
		String gm_soap_pomxml = new File(".").getAbsolutePath() + File.separator + ".." + File.separator
				+ "protocol-pool" + File.separator + "gm-soap" + File.separator + "pom-gm-soap.xml";
		String gm_coap_pomxml = new File(".").getAbsolutePath() + File.separator + ".." + File.separator
				+ "protocol-pool" + File.separator + "gm-coap" + File.separator + "pom.xml";
		// String gm_dpws_pomxml = new File(".").getAbsolutePath() +
		// File.separator + ".." + File.separator + "protocol-pool" +
		// File.separator + "gm-dpws" + File.separator + "pom.xml";

		if (STARTING_FROM_JAR) {

			vsb_manager_pomxml = PathResolver.myFilePath(BcManagerRestService.class, "pom-vsb-manager.xml");
			gm_soap_pomxml = PathResolver.myFilePath(BcManagerRestService.class, "gm-soap/pom-gm-soap.xml");
			gm_coap_pomxml = PathResolver.myFilePath(BcManagerRestService.class, "gm-coap/pom.xml");
			// gm_dpws_pomxml =
			// PathResolver.myFilePath(BcManagerRestService.class,"gm-dpws/pom.xml");
		}
		
		warGenerator.addDependencyFiles(vsb_manager_pomxml);
		
		warGenerator.addDependencyFiles(gm_soap_pomxml);
		
		warGenerator.addDependencyFiles(gm_coap_pomxml);
		
		// warGenerator.addDependencyFiles(gm_dpws_pomxml);
		return warGenerator.generate(busProtocol == ProtocolType.SOAP);

	}

	private void generateBindingComponent(final String interfaceDescription, final ProtocolType busProtocol) {

		copyInterfaceDescription(interfaceDescription);

		BcConfiguration bcConfiguration = null;
		bcConfiguration = new BcConfiguration();

		bcConfiguration.setGeneratedCodePath(Constants.generatedCodePath);

		GmServiceRepresentation gmServiceRepresentation = null;

		String interfaceDescFileExtension = "";
		String[] interfaceDescFileNameComponents = interfaceDescription.split("\\.");
		interfaceDescFileExtension = interfaceDescFileNameComponents[interfaceDescFileNameComponents.length - 1];

		switch (interfaceDescFileExtension) {
		case "gmdl":
			gmServiceRepresentation = ServiceDescriptionParser.getRepresentationFromGMDL(interfaceDescription);
			break;
		case "gidl":
			gmServiceRepresentation = ServiceDescriptionParser.getRepresentationFromGIDL(interfaceDescription);
			break;
		}

		if (busProtocol == ProtocolType.SOAP) {

			bcConfiguration.setTargetNamespace(Constants.target_namespace);
			bcConfiguration.setServiceName(Constants.soap_service_name);

			BcSoapGenerator soapGenerator = (BcSoapGenerator) new BcSoapGenerator(gmServiceRepresentation,
					bcConfiguration).setDebug(true);
			// temporarily disabled
			System.out.println("inria-mimove");
			soapGenerator.generatePOJOAndEndpoint();

			JavaCompiler compiler = ToolProvider.getSystemJavaCompiler();

			if (compiler == null) {
				System.setProperty("java.home", System.getenv("JAVA_HOME"));
				compiler = ToolProvider.getSystemJavaCompiler();
				if (compiler == null) {
					try {
						throw new Exception("Set JAVA_HOME env variable to point to JDK");
					} catch (Exception e) {
						e.printStackTrace();
					}
				}
			}

			StandardJavaFileManager fileManager = compiler.getStandardFileManager(null, Locale.getDefault(), null);

			String strSourceDir = bcConfiguration.getGeneratedCodePath() + File.separator
					+ Constants.target_namespace_path;

			File sourceDir = new File(strSourceDir);

			List<JavaFileObject> javaObjects = scanRecursivelyForJavaFiles(sourceDir, fileManager);

			if (javaObjects.size() == 0) {

				System.out.println("There are no source files to compile in " + sourceDir.getAbsolutePath());
			}

			File classDir = new File(Constants.generatedCodePath);

			// Array of imported classes
			Class[] classes = new Class[] {

					BcManagerRestService.class, BcGmSubcomponent.class, BcGmSubcomponent.class,
					BcRestSubcomponent.class, BcSoapSubcomponent.class, ServiceDescriptionParser.class,
					BcConfiguration.class };

			String[] compileOptions = new String[] { "-d", classDir.getAbsolutePath(), "-cp",
					PathResolver.myClassPath(classes) };
			Iterable<String> compilationOptions = Arrays.asList(compileOptions);
			CompilationTask compilerTask = compiler.getTask(null, fileManager, null, compilationOptions, null,
					javaObjects);

			if (!compilerTask.call()) {

				System.out.println("Could not compile project");

			}

			try {
				for (JavaFileObject jfo : fileManager.list(StandardLocation.CLASS_OUTPUT, "",
						Collections.singleton(JavaFileObject.Kind.CLASS), true)) {

					System.out.println(jfo.getName());
				}
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			soapGenerator.generateWSDL();
			
			
		}
		generateBindingComponentClass(gmServiceRepresentation, busProtocol);
		

	}

	private void copyInterfaceDescription(String interfaceDescription) {
		try {

			File input = new File(interfaceDescription);
			File output = new File(Constants.webapp_src_artifact + File.separator + "config" + File.separator
					+ "serviceDescription.gxdl");
			Scanner sc = new Scanner(input);
			PrintWriter printer = new PrintWriter(output);
			while (sc.hasNextLine()) {
				String s = sc.nextLine();
				printer.write(s);
			}
			sc.close();
			printer.close();
		} catch (FileNotFoundException e) {

			System.err.println("Interface file not found.");
		}
	}

	private static List<JavaFileObject> scanRecursivelyForJavaFiles(File dir, StandardJavaFileManager fileManager) {
		List<JavaFileObject> javaObjects = new LinkedList<JavaFileObject>();
		File[] files = dir.listFiles();
		for (File file : files) {
			if (file.isDirectory()) {

				javaObjects.addAll(scanRecursivelyForJavaFiles(file, fileManager));
			} else if (file.isFile() && file.getName().toLowerCase().endsWith(".java")) {

				javaObjects.add(readJavaObject(file, fileManager));
			}
		}
		return javaObjects;
	}

	private static JavaFileObject readJavaObject(File file, StandardJavaFileManager fileManager) {
		Iterable<? extends JavaFileObject> javaFileObjects = fileManager.getJavaFileObjects(file);
		Iterator<? extends JavaFileObject> it = javaFileObjects.iterator();
		if (it.hasNext()) {
			return it.next();
		}
		throw new RuntimeException("Could not load " + file.getAbsolutePath() + " java file object");
	}

	private static void generateBindingComponentClass(GmServiceRepresentation gmServiceRepresentation,
			ProtocolType busProtocol) {

		String configTemplatePath = "";
		JSONParser parser = new JSONParser();
		JSONObject jsonObject = null;

		configTemplatePath = Constants.configFilePath;

		try {
			jsonObject = (JSONObject) parser.parse(new FileReader(configTemplatePath));
		} catch (IOException | ParseException e) {
			e.printStackTrace();
		}

		/* Creating java code model classes */
		JCodeModel jCodeModel = new JCodeModel();
		/* Adding package here */
		JPackage jp = jCodeModel._package((String) jsonObject.get("target_namespace"));

		/* Giving Class Name to Generate */
		JDefinedClass jc = null;
		try {

			jc = jp._class("BindingComponent");

		} catch (JClassAlreadyExistsException e) {

			e.printStackTrace();
		}

		JClass BcGmSubcomponentClass = jCodeModel
				.ref(eu.chorevolution.vsb.gm.protocols.primitives.BcGmSubcomponent.class);
		JFieldVar ComponentsVar = jc.field(JMod.NONE, BcGmSubcomponentClass.array().array(), "subcomponent");

		JClass GmServiceRepresentationClass = jCodeModel
				.ref(eu.chorevolution.vsb.gmdl.utils.GmServiceRepresentation.class);
		JFieldVar GmServiceRepresentationVar = jc.field(JMod.NONE, GmServiceRepresentationClass,
				"gmServiceRepresentation", JExpr._null());

		JMethod jmCreate = jc.method(JMod.PUBLIC, void.class, "run");

		/* Adding method body */
		JBlock jBlock = jmCreate.body();

		JClass integerClass = jCodeModel.ref(java.lang.Integer.class);
		JClass StringClass = jCodeModel.ref(String.class);

		JVar intFiveVar = jBlock.decl(integerClass, "intFive");
		jBlock.assign(JExpr.ref(intFiveVar.name()), jCodeModel.ref("Integer").staticInvoke("parseInt").arg("5"));

		JVar intOneVar = jBlock.decl(integerClass, "intOne");
		jBlock.assign(JExpr.ref(intOneVar.name()), jCodeModel.ref("Integer").staticInvoke("parseInt").arg("1"));

		JVar intNineVar = jBlock.decl(integerClass, "intNine");
		jBlock.assign(JExpr.ref(intNineVar.name()), jCodeModel.ref("Integer").staticInvoke("parseInt").arg("9"));

		JClass BcManagerClass = null;
		BcManagerClass = jCodeModel.ref(eu.chorevolution.vsb.bc.manager.BcManagerRestService.class);
		JClass ConstantsClass = jCodeModel.ref(eu.chorevolution.vsb.gmdl.utils.Constants.class);

		JClass FileClass = jCodeModel.ref(java.io.File.class);
		JClass BcManagerRestServiceClass = jCodeModel.ref(eu.chorevolution.vsb.bc.manager.BcManagerRestService.class);

		JVar interfaceDescriptionPathVar = null;
		interfaceDescriptionPathVar = jBlock.decl(StringClass, "interfaceDescFilePath");
		jBlock.assign(JExpr.ref(interfaceDescriptionPathVar.name()),
				JExpr._new(FileClass)
						.arg(BcManagerRestServiceClass.dotclass().invoke("getClassLoader").invoke("getResource")
								.arg("example.json").invoke("toExternalForm").invoke("substring").arg(intNineVar))
						.invoke("getParentFile").invoke("getParentFile").invoke("getParentFile").invoke("getParentFile")
						.invoke("getAbsolutePath").plus(FileClass.staticRef("separator"))
						.plus(JExpr._new(StringClass).arg("config")).plus(FileClass.staticRef("separator"))
						.plus(JExpr._new(StringClass).arg("serviceDescription.gxdl")));

		JClass serviceDescriptionClass = jCodeModel.ref(ServiceDescriptionParser.class);

		String interfaceDescriptionPath = Constants.interfaceDescriptionFilePath;
		String extension = "";
		String[] interfaceDescPieces = interfaceDescriptionPath.split("\\.");

		extension = interfaceDescPieces[interfaceDescPieces.length - 1];
		JInvocation getInterfaceRepresentation = null;
		switch (extension) {

		case "gmdl":
			getInterfaceRepresentation = serviceDescriptionClass.staticInvoke("getRepresentationFromGMDL")
					.arg(interfaceDescriptionPathVar);
			break;
		case "gidl":
			getInterfaceRepresentation = serviceDescriptionClass.staticInvoke("getRepresentationFromGIDL")
					.arg(interfaceDescriptionPathVar);
			break;
		}

		jBlock.assign(GmServiceRepresentationVar, getInterfaceRepresentation);

		jBlock.decl(jCodeModel.INT, "num_interfaces",
				GmServiceRepresentationVar.invoke("getInterfaces").invoke("size"));

		jBlock.assign(ComponentsVar, JExpr.ref("new BcGmSubcomponent[num_interfaces][2]"));

		JForLoop forLoop = jBlock._for();
		JVar ivar = forLoop.init(jCodeModel.INT, "i", JExpr.lit(0));
		forLoop.test(ivar.lt(GmServiceRepresentationVar.invoke("getInterfaces").invoke("size")));
		forLoop.update(ivar.assignPlus(JExpr.lit(1)));

		JBlock forBlock = forLoop.body();

		JClass InterfaceClass = jCodeModel.ref(eu.chorevolution.vsb.gmdl.utils.Interface.class);
		JVar InterfaceVar = forBlock.decl(InterfaceClass, "inter", JExpr._null());

		forBlock.assign(JExpr.ref(InterfaceVar.name()),
				GmServiceRepresentationVar.invoke("getInterfaces").invoke("get").arg(ivar));

		JClass RoleTypeClass = jCodeModel.ref(eu.chorevolution.vsb.gmdl.utils.enums.RoleType.class);
		JClass BcRestSubcomponentClass = jCodeModel
				.ref(eu.chorevolution.vsb.gm.protocols.rest.BcRestSubcomponent.class);
		JClass BcSoapSubcomponentClass = jCodeModel
				.ref(eu.chorevolution.vsb.gm.protocols.soap.BcSoapSubcomponent.class);
		JClass BcMQTTSubcomponentClass = jCodeModel
				.ref(eu.chorevolution.vsb.gm.protocols.mqtt.BcMQTTSubcomponent.class);
		JClass BcCoapSubcomponentClass = jCodeModel
				.ref(eu.chorevolution.vsb.gm.protocols.coap.BcCoapSubcomponent.class);

		JClass BcConfigurationClass = jCodeModel.ref(eu.chorevolution.vsb.gmdl.utils.BcConfiguration.class);
		JClass SystemClass = jCodeModel.ref(System.class);

		JClass EnumClass = jCodeModel.ref("eu.chorevolution.vsb.gmdl.utils.enums.RoleType");

		JInvocation printstmt = SystemClass.staticRef("out").invoke("println").arg("genfac start iteration");
		forBlock.add(printstmt);

		JFieldRef RoleTypeServerEnum = null;
		JFieldRef RoleTypeClientEnum = null;

		RoleTypeServerEnum = EnumClass.staticRef("SERVER");
		RoleTypeClientEnum = EnumClass.staticRef("CLIENT");

		JVar RoleTypeClassVar = forBlock.decl(RoleTypeClass, "busRole");

		JConditional roleCondition = forBlock._if(InterfaceVar.invoke("getRole").eq(RoleTypeServerEnum));
		roleCondition._then().assign(JExpr.ref(RoleTypeClassVar.name()), RoleTypeClientEnum);
		roleCondition._else().assign(JExpr.ref(RoleTypeClassVar.name()), RoleTypeServerEnum);

		JVar bcConfig1Var = forBlock.decl(BcConfigurationClass, "bcConfiguration1", JExpr._new(BcConfigurationClass));
		JVar bcConfig2Var = forBlock.decl(BcConfigurationClass, "bcConfiguration2", JExpr._new(BcConfigurationClass));

		JInvocation setRole1 = bcConfig1Var.invoke("setSubcomponentRole").arg(InterfaceVar.invoke("getRole"));
		forBlock.add(setRole1);

		JInvocation setRole2 = bcConfig2Var.invoke("setSubcomponentRole").arg(RoleTypeClassVar);
		forBlock.add(setRole2);

		String packagePath = Constants.target_namespace;
		packagePath = packagePath.replace(".", File.separator);

		String generatedCodePath = Constants.generatedCodePath;
		generatedCodePath = generatedCodePath + File.separator;

		JInvocation parseInvocation1 = bcConfig1Var.invoke("parseFromJSON").arg(GmServiceRepresentationVar).arg(JExpr
				._new(FileClass)
				.arg(BcManagerRestServiceClass.dotclass().invoke("getClassLoader").invoke("getResource")
						.arg("example.json").invoke("toExternalForm").invoke("substring").arg(intNineVar))
				.invoke("getParentFile").invoke("getParentFile").invoke("getParentFile").invoke("getParentFile")
				.invoke("getAbsolutePath").plus(FileClass.staticRef("separator"))
				.plus(JExpr._new(StringClass).arg("config")).plus(FileClass.staticRef("separator"))
				.plus(JExpr._new(StringClass).arg("config_block1_interface_"))
				.plus(jCodeModel.ref(java.lang.String.class).staticInvoke("valueOf").arg(ivar.plus(intOneVar))));
		forBlock.add(parseInvocation1);

		JInvocation parseInvocation2 = bcConfig2Var.invoke("parseFromJSON").arg(GmServiceRepresentationVar).arg(JExpr
				._new(FileClass)
				.arg(BcManagerRestServiceClass.dotclass().invoke("getClassLoader").invoke("getResource")
						.arg("example.json").invoke("toExternalForm").invoke("substring").arg(intNineVar))
				.invoke("getParentFile").invoke("getParentFile").invoke("getParentFile").invoke("getParentFile")
				.invoke("getAbsolutePath").plus(FileClass.staticRef("separator"))
				.plus(JExpr._new(StringClass).arg("config")).plus(FileClass.staticRef("separator"))
				.plus(JExpr._new(StringClass).arg("config_block2_interface_"))
				.plus(jCodeModel.ref(java.lang.String.class).staticInvoke("valueOf").arg(ivar.plus(intOneVar))));
		forBlock.add(parseInvocation2);
		
		switch (busProtocol) {
		case REST:
			for (int i = 1; i <= gmServiceRepresentation.getInterfaces().size(); i++)
				createConfigFile(ProtocolType.REST, Constants.webapp_src_artifact + File.separator + "config"
						+ File.separator + "config_block1_interface_" + String.valueOf(i));
			forBlock.assign(JExpr.ref("subcomponent[i][0]"),
					JExpr._new(BcRestSubcomponentClass).arg(bcConfig1Var).arg(GmServiceRepresentationVar));
			// BcGmSubcomponentVar1.init(JExpr._new(BcRestSubcomponentClass).arg(bcConfig1Var));
			break;
		case SOAP:
			for (int i = 1; i <= gmServiceRepresentation.getInterfaces().size(); i++)
				createConfigFile(ProtocolType.SOAP, Constants.webapp_src_artifact + File.separator + "config"
						+ File.separator + "config_block1_interface_" + String.valueOf(i));
			forBlock.assign(JExpr.ref("subcomponent[i][0]"),
					JExpr._new(BcSoapSubcomponentClass).arg(bcConfig1Var).arg(GmServiceRepresentationVar));
			break;
		case MQTT:
			for (int i = 1; i <= gmServiceRepresentation.getInterfaces().size(); i++)
				createConfigFile(ProtocolType.MQTT, Constants.webapp_src_artifact + File.separator + "config"
						+ File.separator + "config_block1_interface_" + String.valueOf(i));
			forBlock.assign(JExpr.ref("subcomponent[i][0]"),
					JExpr._new(BcMQTTSubcomponentClass).arg(bcConfig1Var).arg(GmServiceRepresentationVar));
			break;
		case COAP:
			for (int i = 1; i <= gmServiceRepresentation.getInterfaces().size(); i++)
				createConfigFile(ProtocolType.COAP, Constants.webapp_src_artifact + File.separator + "config"
						+ File.separator + "config_block1_interface_" + String.valueOf(i));
			forBlock.assign(JExpr.ref("subcomponent[i][0]"),
					JExpr._new(BcCoapSubcomponentClass).arg(bcConfig1Var).arg(GmServiceRepresentationVar));
			break;
		case JMS:
			break;
		case PUB_NUB:
			break;
		case SEMI_SPACE:
			break;
		case WEB_SOCKETS:
			break;
		case ZERO_MQ:
			break;
		default:
			break;
		}
		switch (gmServiceRepresentation.getProtocol()) {
		case REST:
			for (int i = 1; i <= gmServiceRepresentation.getInterfaces().size(); i++)
				createConfigFile(ProtocolType.REST, Constants.webapp_src_artifact + File.separator + "config"
						+ File.separator + "config_block2_interface_" + String.valueOf(i));
			forBlock.assign(JExpr.ref("subcomponent[i][1]"),
					JExpr._new(BcRestSubcomponentClass).arg(bcConfig2Var).arg(GmServiceRepresentationVar));

			break;
		case SOAP:
			for (int i = 1; i <= gmServiceRepresentation.getInterfaces().size(); i++)
				createConfigFile(ProtocolType.SOAP, Constants.webapp_src_artifact + File.separator + "config"
						+ File.separator + "config_block2_interface_" + String.valueOf(i));
			forBlock.assign(JExpr.ref("subcomponent[i][1]"),
					JExpr._new(BcSoapSubcomponentClass).arg(bcConfig2Var).arg(GmServiceRepresentationVar));
			break;
		case MQTT:
			for (int i = 1; i <= gmServiceRepresentation.getInterfaces().size(); i++)
				createConfigFile(ProtocolType.MQTT, Constants.webapp_src_artifact + File.separator + "config"
						+ File.separator + "config_block2_interface_" + String.valueOf(i));
			forBlock.assign(JExpr.ref("subcomponent[i][1]"),
					JExpr._new(BcMQTTSubcomponentClass).arg(bcConfig2Var).arg(GmServiceRepresentationVar));
			break;
		case COAP:
			for (int i = 1; i <= gmServiceRepresentation.getInterfaces().size(); i++)
				createConfigFile(ProtocolType.COAP, Constants.webapp_src_artifact + File.separator + "config"
						+ File.separator + "config_block2_interface_" + String.valueOf(i));
			forBlock.assign(JExpr.ref("subcomponent[i][1]"),
					JExpr._new(BcCoapSubcomponentClass).arg(bcConfig2Var).arg(GmServiceRepresentationVar));
			break;

		case JMS:
			break;
		case PUB_NUB:
			break;
		case SEMI_SPACE:
			break;
		case WEB_SOCKETS:
			break;
		case ZERO_MQ:
			break;
		default:
			break;
		}

		JVar BcGmSubcomponentVar1 = forBlock.decl(BcGmSubcomponentClass, "block1Component",
				JExpr.ref("subcomponent[i][0]"));
		JVar BcGmSubcomponentVar2 = forBlock.decl(BcGmSubcomponentClass, "block2Component",
				JExpr.ref("subcomponent[i][1]"));

		forBlock.add(BcGmSubcomponentVar1.invoke("setNextComponent").arg(BcGmSubcomponentVar2));
		forBlock.add(BcGmSubcomponentVar2.invoke("setNextComponent").arg(BcGmSubcomponentVar1));

		forBlock.add(BcGmSubcomponentVar1.invoke("start"));
		forBlock.add(BcGmSubcomponentVar2.invoke("start"));

		JMethod jmCreatePause = jc.method(JMod.PUBLIC, void.class, "pause");
		JBlock jBlockPause = jmCreatePause.body();

		JForLoop forLoopPause = jBlockPause._for();
		JVar ivarPause = forLoopPause.init(jCodeModel.INT, "i", JExpr.lit(0));
		forLoopPause.test(ivarPause.lt(GmServiceRepresentationVar.invoke("getInterfaces").invoke("size")));
		forLoopPause.update(ivarPause.assignPlus(JExpr.lit(1)));

		JBlock forBlockPause = forLoopPause.body();

		JInvocation printstmtPause = SystemClass.staticRef("out").invoke("println").arg("genfac stop iteration");
		forBlockPause.add(printstmtPause);

		JVar BcGmSubcomponentVar1Pause = forBlockPause.decl(BcGmSubcomponentClass, "block1Component",
				JExpr.ref("subcomponent[i][0]"));
		JVar BcGmSubcomponentVar2Pause = forBlockPause.decl(BcGmSubcomponentClass, "block2Component",
				JExpr.ref("subcomponent[i][1]"));

		forBlockPause.add(BcGmSubcomponentVar1Pause.invoke("stop"));
		forBlockPause.add(BcGmSubcomponentVar2Pause.invoke("stop"));

		try {
			jCodeModel.build(new File(generatedCodePath));
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	private static void createConfigFile(ProtocolType protocol, String filename) {

		String configPath = PathResolver.myFilePath(BcManagerRestService.class, "config.json");
		JSONParser configParser = new JSONParser();
		JSONObject configJsonObject = null;

		String configTemplatePath = "";
		JSONParser parser = new JSONParser();
		JSONObject jsonObject = null;

		switch (protocol) {
		case REST:
			configTemplatePath = PathResolver.myFilePath(BcManagerRestService.class, "rest-config.json");
			break;
		case SOAP:
			configTemplatePath = PathResolver.myFilePath(BcManagerRestService.class, "soap-config.json");
			break;
		case MQTT:
			configTemplatePath = PathResolver.myFilePath(BcManagerRestService.class, "mqtt-config.json");
			break;
		case COAP:
			configTemplatePath = PathResolver.myFilePath(BcManagerRestService.class, "coap-config.json");
			break;
		}

		try {
			configJsonObject = (JSONObject) configParser.parse(new FileReader(configPath));
		} catch (IOException | ParseException e) {
			e.printStackTrace();
		}

		try {

			jsonObject = (JSONObject) parser.parse(new FileReader(configTemplatePath));

		} catch (IOException | ParseException e) {

			e.printStackTrace();
		}

		if (protocol == ProtocolType.SOAP) {

			jsonObject.put("target_namespace", Constants.target_namespace);
			jsonObject.put("service_name", Constants.soap_service_name);
			jsonObject.put("subcomponent_port", Constants.service_port);
			jsonObject.put("bc_rest_service_port", Constants.service_bc_port);
			jsonObject.put("bc_manager_servlet_port", Constants.bc_manager_servlet_port);
			jsonObject.put("subcomponent_address", "http://localhost");
			jsonObject.put("invocation_address", "http://localhost");
		}

		// temporarily disabled
		try (FileWriter file = new FileWriter(filename)) {
			file.write(jsonObject.toJSONString());
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	private static boolean isInterfaceDescriptionFile(String interfaceDescriptionPath) {

		boolean isGoodFile = false;
		File interfaceDescriptionFile = new File(interfaceDescriptionPath);

		if (!interfaceDescriptionFile.isFile()) {

			return isGoodFile;
		}
		String[] interfaceDescription = interfaceDescriptionFile.getName().split("\\.");
		String interfaceDescriptionExtension = interfaceDescription[interfaceDescription.length - 1];

		switch (interfaceDescriptionExtension) {

		case "gmdl":

			isGoodFile = true;
			break;

		case "gidl":

			isGoodFile = true;
			break;
		}
		return isGoodFile;
	}

	private static String interfaceDescriptionBytesToFile(byte[] interfaceDescriptionByteArray) {

		File interfaceDescriptionFile = new File("interfaceDescription.gidl");
		FileOutputStream fos;
		try {

			fos = new FileOutputStream(interfaceDescriptionFile);
			fos.write(interfaceDescriptionByteArray);
			fos.flush();
			fos.close();

		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();

		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return interfaceDescriptionFile.getAbsolutePath();
	}

	private static String deleteSpecialChar(String s) {

		StringBuilder result = new StringBuilder();
		boolean firstChar = true;

		for (char ch : s.toCharArray()) {
			if (firstChar) {
				firstChar = false;
				if (Character.isJavaIdentifierStart(ch)) {
					result.append(ch);
				}
				continue;
			}
			if (Character.isJavaIdentifierPart(ch)) {
				result.append(ch);
			}
		}
		return result.toString();
	}
	/******************************************************************************************************/

}
